﻿

namespace Validation
{
    using System;
    using System.Collections.Generic;
    using System.Globalization;
    using System.Linq;
    using System.Text;
    using System.Threading;
    using System.Threading.Tasks;
    using System.Windows.Controls;

    /// <summary>
    /// This class checks if the display range of an axis is valid:
    /// 
    /// #-------------------------------------------------#
    /// Min                                               Max
    ///          ,--------------------------,
    ///          DisplayMin                DisplayMax
    /// 
    /// DisplayMinRange: Min <= DisplayMin < DisplayMax
    /// DisplayMaxRange: DisplayMin < DisplayMax <= Max
    /// </summary>
    public class AxisDisplayRangeValidationRule : ValidationRule
    {
        public enum CheckDisplayRange
        {
            DisplayRangeMin, // Check the DisplayRangeMin 
            DisplayRangeMax, // DisplayRangeMax
            None
        }

        private CheckDisplayRange m_CheckDisplayRange;

        private AxisSettingsModel m_AxisSettingsModel;

        public AxisDisplayRangeValidationRule()
        {
            m_CheckDisplayRange = CheckDisplayRange.None;

            m_AxisSettingsModel = null;
        }

        public void setAxisSettingsModel(AxisSettingsModel axisSettingsModel)
        {
            m_AxisSettingsModel = axisSettingsModel;
        }

        public void setCheckDisplayRange(CheckDisplayRange checkDisplayRange)
        {
            m_CheckDisplayRange = checkDisplayRange;
        }

        public override ValidationResult Validate(object value, System.Globalization.CultureInfo cultureInfo)
        {
            if (m_AxisSettingsModel == null)
            {
                return ValidationResult.ValidResult;
            }

            if ( m_CheckDisplayRange == CheckDisplayRange.None)
            {
                return ValidationResult.ValidResult;
            }


            if (null == value)
            {
                return ValidationResult.ValidResult;
            }


            double inputNumber = (double)value;

            if (m_CheckDisplayRange == CheckDisplayRange.DisplayRangeMax)
            {
                if ( inputNumber <= m_AxisSettingsModel.ActualMinimumDisplayValue )
                {
                    m_AxisSettingsModel.ActualMaximumDisplayValue = m_AxisSettingsModel.ActualMinimumDisplayValue + 0.1;

                    return new ValidationResult(false, "Wrong");
                }
                else if (inputNumber > m_AxisSettingsModel.ActualMaximumValue)
                {
                    m_AxisSettingsModel.ActualMaximumDisplayValue = m_AxisSettingsModel.ActualMaximumValue;

                    return new ValidationResult(false, "Wrong");
                }
            }
            else if (m_CheckDisplayRange == CheckDisplayRange.DisplayRangeMin)
            {
                if ( inputNumber < m_AxisSettingsModel.ActualMinimumValue )
                {
                    inputNumber = m_AxisSettingsModel.ActualMinimumValue;

                    return new ValidationResult(false, "Wrong");
                }
                else if (inputNumber >= m_AxisSettingsModel.ActualMaximumDisplayValue)
                {
                    inputNumber = m_AxisSettingsModel.ActualMaximumDisplayValue - 0.1;

                    return new ValidationResult(false, "Wrong");
                }
            }

            return ValidationResult.ValidResult;
            
        }
    }
}
